@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "cmd.hpp"
+
 #include "exception.hpp"
 
 #include <osmium/index/map.hpp>
@@ -48,18 +49,20 @@ po::options_description Command::add_common_options(const bool with_progress) {
     return options;
 }
 
-void Command::setup_common(const boost::program_options::variables_map& vm, const po::options_description& desc) {
+bool Command::setup_common(const boost::program_options::variables_map& vm, const po::options_description& desc) {
     if (vm.count("help")) {
         std::cout << "Usage: " << synopsis() << "\n\n"
                   << m_command_factory.get_description(name()) << "\n"
                   << desc
                   << "\nUse 'osmium help " << name() << "' to display the manual page.\n";
-        std::exit(0);
+        return false;
     }
 
     if (vm.count("verbose")) {
         m_vout.verbose(true);
     }
+
+    return true;
 }
 
 void Command::setup_progress(const boost::program_options::variables_map& vm) {
