@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "option_clean.hpp"
+
 #include <osmium/io/file.hpp>
 #include <osmium/io/header.hpp>
 #include <osmium/io/writer_options.hpp>
@@ -66,6 +68,8 @@ protected:
     const CommandFactory& m_command_factory;
     osmium::VerboseOutput m_vout{false};
 
+    OptionClean m_clean;
+
 public:
 
     explicit Command(const CommandFactory& command_factory) :
@@ -106,7 +110,7 @@ public:
     virtual const char* synopsis() const noexcept = 0;
 
     static po::options_description add_common_options(bool with_progress = true);
-    void setup_common(const boost::program_options::variables_map& vm, const po::options_description& desc);
+    bool setup_common(const boost::program_options::variables_map& vm, const po::options_description& desc);
     void setup_progress(const boost::program_options::variables_map& vm);
     void setup_object_type_nwrc(const boost::program_options::variables_map& vm);
     void setup_object_type_nwr(const boost::program_options::variables_map& vm);
@@ -182,9 +186,9 @@ public:
     /// Is any of the input files STDIN?
     bool any_input_is_stdin() const noexcept {
         return std::any_of(m_input_files.cbegin(), m_input_files.cend(),
-            [](const osmium::io::File& file) {
-                return file.filename().empty();
-            });
+                           [](const osmium::io::File& file) {
+                               return file.filename().empty();
+                           });
     }
 
 }; // class with_multiple_osm_inputs
@@ -269,7 +273,7 @@ public:
  */
 class CommandFactory {
 
-    using create_command_type = std::function<Command*()>;
+    using create_command_type = std::function<std::unique_ptr<Command>()>;
 
     struct command_info {
         std::string description; // description of command for help
