@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_apply_changes.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -88,7 +89,9 @@ bool CommandApplyChanges::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_input_file(vm);
     setup_output_file(vm);
@@ -175,8 +178,8 @@ namespace {
 
     public:
 
-        explicit copy_first_with_id(osmium::io::Writer& w) :
-            writer(&w) {
+        explicit copy_first_with_id(osmium::io::Writer* w) :
+            writer(w) {
         }
 
         void operator()(const osmium::OSMObject& obj) {
@@ -192,12 +195,12 @@ namespace {
 
 } // anonymous namespace
 
-static void update_nodes_if_way(osmium::OSMObject& object, const location_index_type& location_index) {
-    if (object.type() != osmium::item_type::way) {
+static void update_nodes_if_way(osmium::OSMObject* object, const location_index_type& location_index) {
+    if (object->type() != osmium::item_type::way) {
         return;
     }
 
-    for (auto& node_ref : static_cast<osmium::Way&>(object).nodes()) {
+    for (auto& node_ref : static_cast<osmium::Way*>(object)->nodes()) {
         auto location = location_index.get_noexcept(node_ref.positive_ref());
         if (location) {
             node_ref.set_location(location);
@@ -333,7 +336,7 @@ bool CommandApplyChanges::run() {
                     auto last_it = it;
                     while (it != objects.end() && osmium::object_order_type_id_reverse_version{}(*it, object)) {
                         if (it->visible()) {
-                            update_nodes_if_way(*it, location_index);
+                            update_nodes_if_way(&*it, location_index);
                             writer(*it);
                         }
                         last_it = it;
@@ -341,14 +344,14 @@ bool CommandApplyChanges::run() {
                     }
 
                     if (last_it == objects.end() || last_it->type() != object.type() || last_it->id() != object.id()) {
-                        update_nodes_if_way(object, location_index);
+                        update_nodes_if_way(&object, location_index);
                         writer(object);
                     }
                 }
             }
             while (it != objects.end()) {
                 if (it->visible()) {
-                    update_nodes_if_way(*it, location_index);
+                    update_nodes_if_way(&*it, location_index);
                     writer(*it);
                 }
                 ++it;
@@ -356,9 +359,7 @@ bool CommandApplyChanges::run() {
         } else {
             m_vout << "Applying changes and writing them to output...\n";
             const auto input = osmium::io::make_input_iterator_range<osmium::OSMObject>(reader);
-            auto output_it = boost::make_function_output_iterator(
-                                copy_first_with_id(writer)
-            );
+            auto output_it = boost::make_function_output_iterator(copy_first_with_id(&writer));
 
             std::set_union(objects.begin(),
                            objects.end(),
