@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_cat.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -33,6 +34,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include <boost/program_options.hpp>
 
+#include <cassert>
 #include <string>
 #include <utility>
 #include <vector>
@@ -67,29 +69,15 @@ bool CommandCat::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_object_type_nwrc(vm);
     setup_input_files(vm);
     setup_output_file(vm);
 
-    if (vm.count("clean")) {
-        for (const auto& c : vm["clean"].as<std::vector<std::string>>()) {
-            if (c == "version") {
-                m_clean_attrs |= clean_options::clean_version;
-            } else if (c == "changeset") {
-                m_clean_attrs |= clean_options::clean_changeset;
-            } else if (c == "timestamp") {
-                m_clean_attrs |= clean_options::clean_timestamp;
-            } else if (c == "uid") {
-                m_clean_attrs |= clean_options::clean_uid;
-            } else if (c == "user") {
-                m_clean_attrs |= clean_options::clean_user;
-            } else {
-                throw argument_error{"Unknown attribute on -c/--clean option: '" + c +"'"};
-            }
-        }
-    }
+    m_clean.setup(vm);
 
     if (vm.count("buffer-data")) {
         m_buffer_data = true;
@@ -104,60 +92,14 @@ void CommandCat::show_arguments() {
 
     m_vout << "  other options:\n";
     show_object_types(m_vout);
-
-    std::string clean_names;
-    if (m_clean_attrs & clean_options::clean_version) {
-        clean_names += "version,";
-    }
-    if (m_clean_attrs & clean_options::clean_changeset) {
-        clean_names += "changeset,";
-    }
-    if (m_clean_attrs & clean_options::clean_timestamp) {
-        clean_names += "timestamp,";
-    }
-    if (m_clean_attrs & clean_options::clean_uid) {
-        clean_names += "uid,";
-    }
-    if (m_clean_attrs & clean_options::clean_user) {
-        clean_names += "user,";
-    }
-
-    if (clean_names.empty()) {
-        clean_names = "(none)";
-    } else {
-        clean_names.resize(clean_names.size() - 1);
-    }
-
-    m_vout << "    attributes to clean: " << clean_names << '\n';
+    m_vout << "    attributes to clean: " << m_clean.to_string() << '\n';
 }
 
-void CommandCat::clean_buffer(osmium::memory::Buffer& buffer) const {
-    for (auto& object : buffer.select<osmium::OSMObject>()) {
-        if (m_clean_attrs & clean_options::clean_version) {
-            object.set_version(static_cast<osmium::object_version_type>(0));
-        }
-        if (m_clean_attrs & clean_options::clean_changeset) {
-            object.set_changeset(static_cast<osmium::changeset_id_type>(0));
-        }
-        if (m_clean_attrs & clean_options::clean_timestamp) {
-            object.set_timestamp(osmium::Timestamp{});
-        }
-        if (m_clean_attrs & clean_options::clean_uid) {
-            object.set_uid(static_cast<osmium::user_id_type>(0));
-        }
-        if (m_clean_attrs & clean_options::clean_user) {
-            object.clear_user();
-        }
-    }
-}
-
-void CommandCat::copy(osmium::ProgressBar& progress_bar, osmium::io::Reader& reader, osmium::io::Writer &writer) const {
+void CommandCat::copy(osmium::ProgressBar& progress_bar, osmium::io::Reader& reader, osmium::io::Writer& writer) const {
     while (osmium::memory::Buffer buffer = reader.read()) {
         progress_bar.update(reader.offset());
 
-        if (m_clean_attrs) {
-            clean_buffer(buffer);
-        }
+        m_clean.apply_to(buffer);
 
         writer(std::move(buffer));
     }
@@ -169,9 +111,7 @@ std::size_t CommandCat::read_buffers(osmium::ProgressBar& progress_bar, osmium::
     while (osmium::memory::Buffer buffer = reader.read()) {
         progress_bar.update(reader.offset());
 
-        if (m_clean_attrs) {
-            clean_buffer(buffer);
-        }
+        m_clean.apply_to(buffer);
 
         size += buffer.committed();
 
@@ -191,18 +131,20 @@ void CommandCat::write_buffers(osmium::ProgressBar& progress_bar, std::vector<os
     }
 }
 
-static void report_filename(osmium::VerboseOutput& vout, const osmium::io::File& file, const osmium::io::Reader& reader) {
+static void report_filename(osmium::VerboseOutput* vout, const osmium::io::File& file, const osmium::io::Reader& reader) {
+    assert(vout);
+
     const auto size = reader.file_size();
-    const auto name = file.filename();
+    const auto& name = file.filename();
 
     if (size == 0) {
         if (name.empty()) {
-            vout << "Reading from stdin...\n";
+            *vout << "Reading from stdin...\n";
         } else {
-            vout << "Reading input file '" << name << "'...\n";
+            *vout << "Reading input file '" << name << "'...\n";
         }
     } else {
-        vout << "Reading input file '" << name << "' (" << size << " bytes)...\n";
+        *vout << "Reading input file '" << name << "' (" << size << " bytes)...\n";
     }
 }
 
@@ -213,10 +155,10 @@ bool CommandCat::run() {
         osmium::io::Reader reader{m_input_files[0], osm_entity_bits()};
         osmium::io::Header header{reader.header()};
 
-        report_filename(m_vout, m_input_files[0], reader);
+        report_filename(&m_vout, m_input_files[0], reader);
 
         setup_header(header);
-        osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+        osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
         if (m_buffer_data) {
             std::vector<osmium::memory::Buffer> buffers;
@@ -248,7 +190,7 @@ bool CommandCat::run() {
             for (const auto& input_file : m_input_files) {
                 progress_bar_reader.remove();
                 osmium::io::Reader reader{input_file, osm_entity_bits()};
-                report_filename(m_vout, input_file, reader);
+                report_filename(&m_vout, input_file, reader);
                 size += read_buffers(progress_bar_reader, reader, buffers);
                 progress_bar_reader.file_done(reader.file_size());
                 reader.close();
@@ -266,7 +208,7 @@ bool CommandCat::run() {
             for (const auto& input_file : m_input_files) {
                 progress_bar.remove();
                 osmium::io::Reader reader{input_file, osm_entity_bits()};
-                report_filename(m_vout, input_file, reader);
+                report_filename(&m_vout, input_file, reader);
                 copy(progress_bar, reader, writer);
                 progress_bar.file_done(reader.file_size());
                 reader.close();
