@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_check_refs.hpp"
+
 #include "util.hpp"
 
 #include <osmium/handler.hpp>
@@ -36,6 +37,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <boost/program_options.hpp>
 
 #include <algorithm>
+#include <cassert>
 #include <cstdint>
 #include <iostream>
 #include <string>
@@ -70,7 +72,9 @@ bool CommandCheckRefs::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_input_file(vm);
 
@@ -109,8 +113,8 @@ class RefCheckHandler : public osmium::handler::Handler {
     uint64_t m_missing_nodes_in_relations = 0;
     uint64_t m_missing_ways_in_relations = 0;
 
-    osmium::VerboseOutput& m_vout;
-    osmium::ProgressBar& m_progress_bar;
+    osmium::VerboseOutput* m_vout;
+    osmium::ProgressBar* m_progress_bar;
     bool m_show_ids;
     bool m_check_relations;
 
@@ -124,11 +128,13 @@ class RefCheckHandler : public osmium::handler::Handler {
 
 public:
 
-    RefCheckHandler(osmium::VerboseOutput& vout, osmium::ProgressBar& progress_bar, bool show_ids, bool check_relations) :
+    RefCheckHandler(osmium::VerboseOutput* vout, osmium::ProgressBar* progress_bar, bool show_ids, bool check_relations) :
         m_vout(vout),
         m_progress_bar(progress_bar),
         m_show_ids(show_ids),
         m_check_relations(check_relations) {
+        assert(vout);
+        assert(progress_bar);
     }
 
     uint64_t node_count() const noexcept {
@@ -181,8 +187,8 @@ public:
         m_check_order.node(node);
 
         if (m_node_count == 0) {
-            m_progress_bar.remove();
-            m_vout << "Reading nodes...\n";
+            m_progress_bar->remove();
+            *m_vout << "Reading nodes...\n";
         }
         ++m_node_count;
 
@@ -193,8 +199,8 @@ public:
         m_check_order.way(way);
 
         if (m_way_count == 0) {
-            m_progress_bar.remove();
-            m_vout << "Reading ways...\n";
+            m_progress_bar->remove();
+            *m_vout << "Reading ways...\n";
         }
         ++m_way_count;
 
@@ -216,8 +222,8 @@ public:
         m_check_order.relation(relation);
 
         if (m_relation_count == 0) {
-            m_progress_bar.remove();
-            m_vout << "Reading relations...\n";
+            m_progress_bar->remove();
+            *m_vout << "Reading relations...\n";
         }
         ++m_relation_count;
 
@@ -276,7 +282,7 @@ public:
 bool CommandCheckRefs::run() {
     osmium::io::Reader reader{m_input_file};
     osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
-    RefCheckHandler handler{m_vout, progress_bar, m_show_ids, m_check_relations};
+    RefCheckHandler handler{&m_vout, &progress_bar, m_show_ids, m_check_relations};
 
     while (osmium::memory::Buffer buffer = reader.read()) {
         progress_bar.update(reader.offset());
@@ -307,7 +313,7 @@ bool CommandCheckRefs::run() {
         std::cerr << "Nodes in ways missing: " << handler.missing_nodes_in_ways() << "\n";
     }
 
-    m_vout << "Memory used for indexes: " << (handler.used_memory() / (1024 * 1024)) << " MBytes\n";
+    m_vout << "Memory used for indexes: " << show_mbytes(handler.used_memory()) << " MBytes\n";
 
     show_memory_used();
     m_vout << "Done.\n";
