@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_diff.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -81,7 +82,9 @@ bool CommandDiff::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_object_type_nwr(vm);
     setup_input_files(vm);
@@ -166,16 +169,16 @@ public:
     OutputAction(OutputAction&&) noexcept = delete;
     OutputAction& operator=(OutputAction&&) noexcept = delete;
 
-    virtual void left(osmium::OSMObject& /* object */) {
+    virtual void left(const osmium::OSMObject& /* object */) {
     }
 
-    virtual void right(osmium::OSMObject& /* object */) {
+    virtual void right(const osmium::OSMObject& /* object */) {
     }
 
-    virtual void same(osmium::OSMObject& /* object */) {
+    virtual void same(const osmium::OSMObject& /* object */) {
     }
 
-    virtual void different(osmium::OSMObject& /* left */, osmium::OSMObject& /* right */) {
+    virtual void different(const osmium::OSMObject& /* left */, const osmium::OSMObject& /* right */) {
     }
 
 }; // class OutputAction
@@ -198,19 +201,19 @@ public:
         m_fd(fd) {
     }
 
-    void left(osmium::OSMObject& object) override {
+    void left(const osmium::OSMObject& object) override {
         print('-', object);
     }
 
-    void right(osmium::OSMObject& object) override {
+    void right(const osmium::OSMObject& object) override {
         print('+', object);
     }
 
-    void same(osmium::OSMObject& object) override {
+    void same(const osmium::OSMObject& object) override {
         print(' ', object);
     }
 
-    void different(osmium::OSMObject& left, osmium::OSMObject& /* right */) override {
+    void different(const osmium::OSMObject& left, const osmium::OSMObject& /* right */) override {
         print('*', left);
     }
 
@@ -226,19 +229,19 @@ public:
         m_writer(file, ow) {
     }
 
-    void left(osmium::OSMObject& object) override {
+    void left(const osmium::OSMObject& object) override {
         m_writer(object);
     }
 
-    void right(osmium::OSMObject& object) override {
+    void right(const osmium::OSMObject& object) override {
         m_writer(object);
     }
 
-    void same(osmium::OSMObject& object) override {
+    void same(const osmium::OSMObject& object) override {
         m_writer(object);
     }
 
-    void different(osmium::OSMObject& left, osmium::OSMObject& right) override {
+    void different(const osmium::OSMObject& left, const osmium::OSMObject& right) override {
         m_writer(left);
         m_writer(right);
     }
@@ -259,10 +262,10 @@ bool CommandDiff::run() {
 
     if (m_output_action == "compact") {
         const int fd = osmium::io::detail::open_for_writing(m_output_filename, m_output_overwrite);
-        action.reset(new OutputActionCompact{fd});
+        action = std::make_unique<OutputActionCompact>(fd);
     } else if (m_output_action == "osm") {
         m_output_file.set("diff");
-        action.reset(new OutputActionOSM{m_output_file, m_output_overwrite});
+        action = std::make_unique<OutputActionOSM>(m_output_file, m_output_overwrite);
     }
 
     uint64_t count_left = 0;
