@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_export.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -169,7 +170,7 @@ static Ruleset parse_tags_ruleset(const rapidjson::Value& object, const char* ke
     return ruleset;
 }
 
-static bool parse_string_array(const rapidjson::Value& object, const char* key, std::vector<std::string>& result) {
+static bool parse_string_array(const rapidjson::Value& object, const char* key, std::vector<std::string>* result) {
     const auto json = object.FindMember(key);
     if (json == object.MemberEnd()) {
         return false;
@@ -185,7 +186,7 @@ static bool parse_string_array(const rapidjson::Value& object, const char* key,
         }
 
         if (value.GetString()[0] != '\0') {
-            result.emplace_back(value.GetString());
+            result->emplace_back(value.GetString());
         }
     }
 
@@ -228,8 +229,8 @@ void CommandExport::parse_config_file() {
         m_area_ruleset.set_rule_type(tags_filter_rule_type::any);
     }
 
-    parse_string_array(doc, "include_tags", m_include_tags);
-    parse_string_array(doc, "exclude_tags", m_exclude_tags);
+    parse_string_array(doc, "include_tags", &m_include_tags);
+    parse_string_array(doc, "exclude_tags", &m_exclude_tags);
 }
 
 void CommandExport::canonicalize_output_format() {
@@ -329,7 +330,9 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
         return false;
     }
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_input_file(vm);
 
@@ -490,26 +493,28 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     return true;
 }
 
-static void print_taglist(osmium::VerboseOutput& vout, const std::vector<std::string>& strings) {
+static void print_taglist(osmium::VerboseOutput* vout, const std::vector<std::string>& strings) {
+    assert(vout);
     for (const auto& str : strings) {
-        vout << "    " << str << '\n';
+        *vout << "    " << str << '\n';
     }
 }
 
-static void print_ruleset(osmium::VerboseOutput& vout, const Ruleset& ruleset) {
+static void print_ruleset(osmium::VerboseOutput* vout, const Ruleset& ruleset) {
+    assert(vout);
     switch (ruleset.rule_type()) {
         case tags_filter_rule_type::none:
-            vout << "none\n";
+            *vout << "none\n";
             break;
         case tags_filter_rule_type::any:
-            vout << "any\n";
+            *vout << "any\n";
             break;
         case tags_filter_rule_type::list:
-            vout << "one of the following:\n";
+            *vout << "one of the following:\n";
             print_taglist(vout, ruleset.tags());
             break;
         case tags_filter_rule_type::other:
-            vout << "if other tag list doesn't match\n";
+            *vout << "if other tag list doesn't match\n";
             break;
     }
 }
@@ -553,16 +558,16 @@ void CommandExport::show_arguments() {
     }
 
     m_vout << "  linear tags: ";
-    print_ruleset(m_vout, m_linear_ruleset);
+    print_ruleset(&m_vout, m_linear_ruleset);
     m_vout << "  area tags:   ";
-    print_ruleset(m_vout, m_area_ruleset);
+    print_ruleset(&m_vout, m_area_ruleset);
 
     if (!m_include_tags.empty()) {
         m_vout << "  include only these tags:\n";
-        print_taglist(m_vout, m_include_tags);
+        print_taglist(&m_vout, m_include_tags);
     } else if (!m_exclude_tags.empty()) {
         m_vout << "  exclude these tags:\n";
-        print_taglist(m_vout, m_exclude_tags);
+        print_taglist(&m_vout, m_exclude_tags);
     }
 
     m_vout << "  other options:\n";
@@ -577,19 +582,19 @@ static std::unique_ptr<ExportFormat> create_handler(const std::string& output_fo
                                                     osmium::io::fsync fsync,
                                                     const options_type& options) {
     if (output_format == "geojson" || output_format == "geojsonseq") {
-        return std::unique_ptr<ExportFormat>{new ExportFormatJSON{output_format, output_filename, overwrite, fsync, options}};
+        return std::make_unique<ExportFormatJSON>(output_format, output_filename, overwrite, fsync, options);
     }
 
     if (output_format == "pg") {
-        return std::unique_ptr<ExportFormat>{new ExportFormatPg{output_format, output_filename, overwrite, fsync, options}};
+        return std::make_unique<ExportFormatPg>(output_format, output_filename, overwrite, fsync, options);
     }
 
     if (output_format == "text") {
-        return std::unique_ptr<ExportFormat>{new ExportFormatText{output_format, output_filename, overwrite, fsync, options}};
+        return std::make_unique<ExportFormatText>(output_format, output_filename, overwrite, fsync, options);
     }
 
     if (output_format == "spaten") {
-        return std::unique_ptr<ExportFormat>{new ExportFormatSpaten{output_format, output_filename, overwrite, fsync, options}};
+        return std::make_unique<ExportFormatSpaten>(output_format, output_filename, overwrite, fsync, options);
     }
 
     throw argument_error{"Unknown output format"};
@@ -626,7 +631,9 @@ bool CommandExport::run() {
         auto location_index_pos = map_factory.create_map(m_index_type_name);
         auto location_index_neg = map_factory.create_map(m_index_type_name);
         location_handler_type location_handler{*location_index_pos, *location_index_neg};
-        location_handler.ignore_errors();
+        if (!m_stop_on_error) {
+            location_handler.ignore_errors();
+        }
 
         osmium::io::ReaderWithProgressBar reader{display_progress(), m_input_filename};
         osmium::apply(reader, check_order_handler, location_handler, export_handler, mp_manager.handler([&export_handler](osmium::memory::Buffer&& buffer) {
@@ -634,9 +641,17 @@ bool CommandExport::run() {
         }));
         reader.close();
         m_vout << "About "
-               << ((location_index_pos->used_memory() + location_index_neg->used_memory()) / (1024 * 1024))
+               << show_mbytes(location_index_pos->used_memory() + location_index_neg->used_memory())
                << " MBytes used for node location index (in main memory or on disk).\n";
     }
+
+    if (m_stop_on_error) {
+        const auto incomplete_relations = mp_manager.relations_database().count_relations();
+        if (incomplete_relations > 0) {
+            throw osmium::geometry_error{"Found " + std::to_string(incomplete_relations) + " incomplete relation(s)"};
+        }
+    }
+
     m_vout << "Second pass done.\n";
     export_handler.close();
 
