@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_extract.hpp"
+
 #include "exception.hpp"
 
 #include "extract/extract_bbox.hpp"
@@ -164,7 +165,9 @@ static osmium::Box parse_bbox(const rapidjson::Value& value) {
     throw config_error{"'bbox' member is not an array or object."};
 }
 
-static std::size_t parse_multipolygon_object(const std::string& directory, std::string file_name, std::string file_type, osmium::memory::Buffer& buffer) {
+static std::size_t parse_multipolygon_object(const std::string& directory, std::string file_name, std::string file_type, osmium::memory::Buffer* buffer) {
+    assert(buffer);
+
     if (file_name.empty()) {
         throw config_error{"Missing 'file_name' in '(multi)polygon' object."};
     }
@@ -192,7 +195,7 @@ static std::size_t parse_multipolygon_object(const std::string& directory, std::
 
     if (file_type == "osm") {
         try {
-            OSMFileParser parser{buffer, file_name};
+            OSMFileParser parser{*buffer, file_name};
             return parser();
         } catch (const std::system_error& e) {
             throw osmium::io_error{std::string{"While reading file '"} + file_name + "':\n" + e.what()};
@@ -202,14 +205,14 @@ static std::size_t parse_multipolygon_object(const std::string& directory, std::
             throw osmium::io_error{std::string{"While reading file '"} + file_name + "':\n" + e.what()};
         }
     } else if (file_type == "geojson") {
-        GeoJSONFileParser parser{buffer, file_name};
+        GeoJSONFileParser parser{*buffer, file_name};
         try {
             return parser();
         } catch (const config_error& e) {
             throw geojson_error{e.what()};
         }
     } else if (file_type == "poly") {
-        PolyFileParser parser{buffer, file_name};
+        PolyFileParser parser{*buffer, file_name};
         return parser();
     } else if (file_type.empty()) {
         throw config_error{"Could not autodetect file type in '(multi)polygon' object. Add a 'file_type'."};
@@ -218,13 +221,17 @@ static std::size_t parse_multipolygon_object(const std::string& directory, std::
     throw config_error{std::string{"Unknown file type: '"} + file_type + "' in '(multi)polygon.file_type'"};
 }
 
-static std::size_t parse_multipolygon_object(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer& buffer) {
+static std::size_t parse_multipolygon_object(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer* buffer) {
+    assert(buffer);
+
     const std::string file_name{get_value_as_string(value, "file_name")};
     const std::string file_type{get_value_as_string(value, "file_type")};
     return parse_multipolygon_object(directory, file_name, file_type, buffer);
 }
 
-static std::size_t parse_polygon(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer& buffer) {
+static std::size_t parse_polygon(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer* buffer) {
+    assert(buffer);
+
     if (value.IsArray()) {
         return parse_polygon_array(value, buffer);
     }
@@ -236,7 +243,9 @@ static std::size_t parse_polygon(const std::string& directory, const rapidjson::
     throw config_error{"Polygon must be an object or array."};
 }
 
-std::size_t parse_multipolygon(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer& buffer) {
+std::size_t parse_multipolygon(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer* buffer) {
+    assert(buffer);
+
     if (value.IsArray()) {
         return parse_multipolygon_array(value, buffer);
     }
@@ -340,11 +349,11 @@ void CommandExtract::parse_config_file() {
             }
 
             if (json_bbox != e.MemberEnd()) {
-                m_extracts.emplace_back(new ExtractBBox{output_file, description, parse_bbox(json_bbox->value)});
+                m_extracts.push_back(std::make_unique<ExtractBBox>(output_file, description, parse_bbox(json_bbox->value)));
             } else if (json_polygon != e.MemberEnd()) {
-                m_extracts.emplace_back(new ExtractPolygon{output_file, description, m_buffer, parse_polygon(m_config_directory, json_polygon->value, m_buffer)});
+                m_extracts.push_back(std::make_unique<ExtractPolygon>(output_file, description, m_buffer, parse_polygon(m_config_directory, json_polygon->value, &m_buffer)));
             } else if (json_multipolygon != e.MemberEnd()) {
-                m_extracts.emplace_back(new ExtractPolygon{output_file, description, m_buffer, parse_multipolygon(m_config_directory, json_multipolygon->value, m_buffer)});
+                m_extracts.push_back(std::make_unique<ExtractPolygon>(output_file, description, m_buffer, parse_multipolygon(m_config_directory, json_multipolygon->value, &m_buffer)));
             } else {
                 throw config_error{"Missing geometry for extract. Need 'bbox', 'polygon', or 'multipolygon'."};
             }
@@ -401,21 +410,21 @@ std::unique_ptr<ExtractStrategy> CommandExtract::make_strategy(const std::string
         if (m_with_history) {
             throw argument_error{"The 'simple' strategy is not supported for history files."};
         }
-        return std::unique_ptr<ExtractStrategy>(new strategy_simple::Strategy{m_extracts, m_options});
+        return std::make_unique<strategy_simple::Strategy>(m_extracts, m_options);
     }
 
     if (name == "complete_ways") {
         if (m_with_history) {
-            return std::unique_ptr<ExtractStrategy>(new strategy_complete_ways_with_history::Strategy{m_extracts, m_options});
+            return std::make_unique<strategy_complete_ways_with_history::Strategy>(m_extracts, m_options);
         }
-        return std::unique_ptr<ExtractStrategy>(new strategy_complete_ways::Strategy{m_extracts, m_options});
+        return std::make_unique<strategy_complete_ways::Strategy>(m_extracts, m_options);
     }
 
     if (name == "smart") {
         if (m_with_history) {
             throw argument_error{"The 'smart' strategy is not supported for history files."};
         }
-        return std::unique_ptr<ExtractStrategy>(new strategy_smart::Strategy{m_extracts, m_options});
+        return std::make_unique<strategy_smart::Strategy>(m_extracts, m_options);
     }
 
     throw argument_error{std::string{"Unknown extract strategy: '"} + name + "'."};
@@ -432,6 +441,7 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
     ("strategy,s", po::value<std::string>()->default_value("complete_ways"), "Use named extract strategy")
     ("with-history,H", "Input file and output files are history files")
     ("set-bounds", "Sets bounds (bounding box) in header")
+    ("clean", po::value<std::vector<std::string>>(), "Clean attribute (version, changeset, timestamp, uid, user)")
     ;
 
     po::options_description opts_common{add_common_options()};
@@ -456,11 +466,15 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_input_file(vm);
     init_output_file(vm);
 
+    m_clean.setup(vm);
+
     if (vm.count("config") + vm.count("bbox") + vm.count("polygon") > 1) {
         throw argument_error{"Can only use one of --config/-c, --bbox/-b, or --polygon/-p."};
     }
@@ -495,7 +509,7 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
         if (m_with_history) {
             m_output_file.set_has_multiple_object_versions(true);
         }
-        m_extracts.emplace_back(new ExtractBBox{m_output_file, "", parse_bbox(vm["bbox"].as<std::string>(), "--box/-b")});
+        m_extracts.push_back(std::make_unique<ExtractBBox>(m_output_file, "", parse_bbox(vm["bbox"].as<std::string>(), "--box/-b")));
     }
 
     if (vm.count("polygon")) {
@@ -506,7 +520,7 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
         if (m_with_history) {
             m_output_file.set_has_multiple_object_versions(true);
         }
-        m_extracts.emplace_back(new ExtractPolygon{m_output_file, "", m_buffer, parse_multipolygon_object("./", vm["polygon"].as<std::string>(), "", m_buffer)});
+        m_extracts.push_back(std::make_unique<ExtractPolygon>(m_output_file, "", m_buffer, parse_multipolygon_object("./", vm["polygon"].as<std::string>(), "", &m_buffer)));
     }
 
     if (vm.count("option")) {
@@ -537,6 +551,7 @@ void CommandExtract::show_arguments() {
     m_vout << "  other options:\n";
     m_vout << "    config file: " << m_config_file_name << '\n';
     m_vout << "    output directory: " << m_output_directory << '\n';
+    m_vout << "    attributes to clean: " << m_clean.to_string() << '\n';
 
     m_vout << '\n';
 }
@@ -614,7 +629,7 @@ bool CommandExtract::run() {
             file_header.add_box(extract->envelope());
         }
         init_header(file_header, input_header, extract->header_options());
-        extract->open_file(file_header, m_output_overwrite, m_fsync);
+        extract->open_file(file_header, m_output_overwrite, m_fsync, &m_clean);
     }
 
     m_strategy->run(m_vout, display_progress(), m_input_file);
