@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_fileinfo.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -300,14 +301,13 @@ public:
             std::cout << '\n';
         }
 
-        const auto buffers_size = static_cast<double>(info_handler.buffers_size / (1000 * 1000)) / 1000; // NOLINT(bugprone-integer-division)
-        std::cout << "  Sum of buffer sizes: " << info_handler.buffers_size << " (" << buffers_size << " GB)\n";
-
-        const auto buffers_capacity= static_cast<double>(info_handler.buffers_capacity / (1000 * 1000)) / 1000; // NOLINT(bugprone-integer-division)
+        std::cout << "  Sum of buffer sizes: " << info_handler.buffers_size
+                  << " (" << show_gbytes(info_handler.buffers_size) << " GB)\n";
 
         if (info_handler.buffers_capacity != 0) {
             const auto fill_factor = std::round(100 * static_cast<double>(info_handler.buffers_size) / static_cast<double>(info_handler.buffers_capacity));
-            std::cout << "  Sum of buffer capacities: " << info_handler.buffers_capacity << " (" << buffers_capacity << " GB, " << fill_factor << "% full)\n";
+            std::cout << "  Sum of buffer capacities: " << info_handler.buffers_capacity
+                      << " (" << show_gbytes(info_handler.buffers_capacity) << " GB, " << fill_factor << "% full)\n";
         } else {
             std::cout << "  Sum of buffer capacities: 0 (0 GB)\n";
         }
@@ -751,7 +751,9 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_object_type_nwrc(vm);
 
@@ -858,11 +860,11 @@ void CommandFileinfo::show_arguments() {
 bool CommandFileinfo::run() {
     std::unique_ptr<Output> output;
     if (m_json_output) {
-        output.reset(new JSONOutput{});
+        output = std::make_unique<JSONOutput>();
     } else if (m_get_value.empty()) {
-        output.reset(new HumanReadableOutput{});
+        output = std::make_unique<HumanReadableOutput>();
     } else {
-        output.reset(new SimpleOutput{m_get_value});
+        output = std::make_unique<SimpleOutput>(m_get_value);
     }
 
     output->set_crc(m_calculate_crc);
