@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_merge.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -74,7 +75,9 @@ bool CommandMerge::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_input_files(vm);
     setup_output_file(vm);
@@ -110,7 +113,7 @@ namespace {
     public:
 
         explicit DataSource(const osmium::io::File& file, bool with_history) :
-            m_reader(new osmium::io::Reader{file}),
+            m_reader(std::make_unique<osmium::io::Reader>(file)),
             m_name(file.filename()),
             m_iterator(*m_reader),
             m_warning(!with_history) {
