@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_query_locations_index.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -69,7 +70,9 @@ bool CommandQueryLocationsIndex::setup(const std::vector<std::string>& arguments
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     init_output_file(vm);
 
     if (vm.count("index-file")) {
@@ -121,7 +124,7 @@ bool CommandQueryLocationsIndex::run() {
     osmium::index::map::DenseFileArray<osmium::unsigned_object_id_type, osmium::Location> location_index{fd};
 
     if (m_dump) {
-        const std::size_t max_buffer_size = 11 * 1024 * 1024;
+        const std::size_t max_buffer_size = 11UL * 1024UL * 1024UL;
         osmium::memory::Buffer buffer{max_buffer_size};
 
         m_vout << "Opening output file...\n";
@@ -137,7 +140,7 @@ bool CommandQueryLocationsIndex::run() {
                 builder.set_location(location_index.get_noexcept(i));
             }
             buffer.commit();
-            if (buffer.committed() > 10 * 1024 * 1024) {
+            if (buffer.committed() > 10UL * 1024UL * 1024UL) {
                 writer(std::move(buffer));
                 buffer = osmium::memory::Buffer{max_buffer_size};
             }
