@@ -2,7 +2,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_renumber.hpp"
+
 #include "exception.hpp"
 
 #include <osmium/io/detail/read_write.hpp>
@@ -40,7 +41,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include <algorithm>
 #include <cerrno>
-#include <cstring>
 #include <fcntl.h>
 #include <fstream>
 #include <iostream>
@@ -49,6 +49,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <string>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <system_error>
 #include <utility>
 #include <vector>
 
@@ -214,7 +215,9 @@ bool CommandRenumber::setup(const std::vector<std::string>& arguments) {
         return false;
     }
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_object_type_nwr(vm);
     setup_input_file(vm);
@@ -295,7 +298,7 @@ void CommandRenumber::read_index(osmium::item_type type) {
         if (errno == ENOENT) {
             return;
         }
-        throw std::runtime_error{std::string{"Could not open file '"} + f + "': " + std::strerror(errno)};
+        throw std::system_error{errno, std::system_category(), "Could not open file '" + f + "'"};
     }
 #ifdef _WIN32
     _setmode(fd, _O_BINARY);
@@ -320,7 +323,7 @@ void CommandRenumber::write_index(osmium::item_type type) {
     const std::string f{filename(osmium::item_type_to_name(type))};
     const int fd = ::open(f.c_str(), O_WRONLY | O_CREAT, 0666); // NOLINT(hicpp-signed-bitwise)
     if (fd < 0) {
-        throw std::runtime_error{std::string{"Could not open file '"} + f + "': " + std::strerror(errno)};
+        throw std::system_error{errno, std::system_category(), "Could not open file '" + f + "'"};
     }
 #ifdef _WIN32
     _setmode(fd, _O_BINARY);
@@ -331,12 +334,12 @@ void CommandRenumber::write_index(osmium::item_type type) {
     close(fd);
 }
 
-void read_relations(const osmium::io::File& input_file, id_map& map) {
+void read_relations(const osmium::io::File& input_file, id_map* map) {
     osmium::io::Reader reader{input_file, osmium::osm_entity_bits::relation};
 
     const auto input = osmium::io::make_input_iterator_range<osmium::Relation>(reader);
     for (const osmium::Relation& relation : input) {
-        map(relation.id());
+        (*map)(relation.id());
     }
 
     reader.close();
@@ -367,7 +370,7 @@ bool CommandRenumber::run() {
 
     if (osm_entity_bits() & osmium::osm_entity_bits::relation) {
         m_vout << "First pass (of two) through input file (reading relations)...\n";
-        read_relations(m_input_file, m_id_map(osmium::item_type::relation));
+        read_relations(m_input_file, &m_id_map(osmium::item_type::relation));
         m_vout << "First pass done.\n";
         m_vout << "Second pass (of two) through input file...\n";
     } else {
