@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_show.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -98,7 +99,9 @@ bool CommandShow::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_object_type_nwrc(vm);
     setup_input_file(vm);
 
@@ -163,10 +166,10 @@ static int execute_pager(const std::string& pager, bool with_color) {
         ::close(pipefd[1]); // close write end of the pipe
         ::close(0); // close stdin
         if (::dup2(pipefd[0], 0) < 0) { // put end of pipe as stdin
-            std::exit(1);
+            throw std::system_error{errno, std::system_category(), "Could not run pager: dup2() call failed"};
         }
 
-        if (with_color && pager.substr(pager.size() - 4, 4) == "less") {
+        if (with_color && pager.size() >= 4 && pager.substr(pager.size() - 4, 4) == "less") {
             ::execlp(pager.c_str(), pager.c_str(), "-R", nullptr);
         } else {
             // execute pager without arguments
@@ -175,16 +178,12 @@ static int execute_pager(const std::string& pager, bool with_color) {
 
         // Exec will either succeed and never return here, or it fails and
         // we'll exit.
-        std::exit(1);
+        throw std::system_error{errno, std::system_category(), "Could not run pager: execlp() call failed"};
     }
 
     // parent
     ::close(pipefd[0]); // close read end of the pipe
 
-    if (signal(SIGPIPE, SIG_IGN) == SIG_ERR) { // NOLINT(cppcoreguidelines-pro-type-cstyle-cast)
-        throw std::system_error{errno, std::system_category(), "Could not run pager: signal() call failed"};
-    }
-
     return pipefd[1];
 }
 #endif
