@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "command_tags_count.hpp"
+
 #include "exception.hpp"
 #include "util.hpp"
 
@@ -103,7 +104,6 @@ bool CommandTagsCount::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
     opts_cmd.add_options()
     ("expressions,e", po::value<std::string>(), "Read tag expressions from file")
-//    ("output-format,f", po::value<std::string>(), "Format of output file")
     ("min-count,m", po::value<counter_type>(), "Min count shown (default: 0)")
     ("max-count,M", po::value<counter_type>(), "Max count shown (default: none)")
     ("output,o", po::value<std::string>(), "Output file (default: stdout)")
@@ -135,7 +135,9 @@ bool CommandTagsCount::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
-    setup_common(vm, desc);
+    if (!setup_common(vm, desc)) {
+        return false;
+    }
     setup_progress(vm);
     setup_object_type_nwr(vm);
     setup_input_file(vm);
@@ -144,10 +146,6 @@ bool CommandTagsCount::setup(const std::vector<std::string>& arguments) {
         m_output_filename = vm["output"].as<std::string>();
     }
 
-/*    if (vm.count("output-format")) {
-        m_output_format = vm["output-format"].as<std::string>();
-    }*/
-
     if (vm.count("overwrite")) {
         m_output_overwrite = osmium::io::overwrite::allow;
     }
@@ -191,7 +189,6 @@ void CommandTagsCount::show_arguments() {
 
     m_vout << "  output options:\n";
     m_vout << "    file name: " << m_output_filename << '\n';
-//    m_vout << "    file format: " << m_output_format << '\n';
     m_vout << "    overwrite: " << yes_no(m_output_overwrite == osmium::io::overwrite::allow);
     m_vout << "  other options:\n";
     m_vout << "    sort order: " << m_sort_order << '\n';
@@ -219,30 +216,30 @@ std::vector<element_type> CommandTagsCount::sort_results() const {
     return results;
 }
 
-static void append_escaped(std::string& out, const char* str) {
-    out += '"';
+static void append_escaped(std::string* out, const char* str) {
+    *out += '"';
     for (; *str != '\0'; ++str) {
         if (*str == '"') {
-            out += '"';
+            *out += '"';
         }
-        out += *str;
+        *out += *str;
     }
-    out += '"';
+    *out += '"';
 }
 
 static void write_results(const std::vector<element_type>& results, int fd) {
     std::string out;
-    const std::size_t buffer_size = 1024 * 1024;
+    const std::size_t buffer_size = 1024UL * 1024UL;
     out.reserve(buffer_size);
 
     for (const auto& c : results) {
         out += std::to_string(c.count);
         out += '\t';
-        append_escaped(out, c.name->key());
+        append_escaped(&out, c.name->key());
         const char* value = c.name->value();
         if (value) {
             out += '\t';
-            append_escaped(out, value);
+            append_escaped(&out, value);
         }
         out += '\n';
         if (out.size() > (buffer_size - 1000)) {
@@ -258,7 +255,7 @@ static void write_results(const std::vector<element_type>& results, int fd) {
 
 bool CommandTagsCount::run() {
     m_vout << "Opening input file...\n";
-    osmium::io::Reader reader{m_input_file, osm_entity_bits()};
+    osmium::io::Reader reader{m_input_file, osm_entity_bits(), osmium::io::read_meta::no};
 
     m_vout << "Opening output file...\n";
     int fd = 1;
