@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,18 +20,19 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "export_format_json.hpp"
+
 #include "../exception.hpp"
 #include "../util.hpp"
-#include "export_format_json.hpp"
 
 #include <osmium/io/detail/read_write.hpp>
 
-static constexpr const std::size_t initial_buffer_size = 1024 * 1024;
-static constexpr const std::size_t flush_buffer_size   =  800 * 1024;
+static constexpr const std::size_t initial_buffer_size = 1024UL * 1024UL;
+static constexpr const std::size_t flush_buffer_size   =  800UL * 1024UL;
 
-static void add_to_stream(rapidjson::StringBuffer& stream, const char* s) {
+static void add_to_stream(rapidjson::StringBuffer* stream, const char* s) {
     while (*s) {
-        stream.Put(*s++);
+        stream->Put(*s++);
     }
 }
 
@@ -49,7 +50,7 @@ ExportFormatJSON::ExportFormatJSON(const std::string& output_format,
     m_factory(m_writer) {
     m_stream.Reserve(initial_buffer_size);
     if (!m_text_sequence_format) {
-        add_to_stream(m_stream, "{\"type\":\"FeatureCollection\",\"features\":[\n");
+        add_to_stream(&m_stream, "{\"type\":\"FeatureCollection\",\"features\":[\n");
     }
     m_committed_size = m_stream.GetSize();
 
@@ -193,9 +194,9 @@ void ExportFormatJSON::close() {
     if (m_fd > 0) {
         rollback_uncomitted();
 
-        add_to_stream(m_stream, "\n");
+        add_to_stream(&m_stream, "\n");
         if (!m_text_sequence_format) {
-            add_to_stream(m_stream, "]}\n");
+            add_to_stream(&m_stream, "]}\n");
         }
 
         flush_to_output();
