@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "export_format_pg.hpp"
+
 #include "../exception.hpp"
 #include "../util.hpp"
-#include "export_format_pg.hpp"
 
 #include <osmium/io/detail/read_write.hpp>
 #include <osmium/io/detail/string_util.hpp>
@@ -185,21 +186,21 @@ bool ExportFormatPg::add_tags_json(const osmium::OSMObject& object) {
     return has_tags;
 }
 
-static void add_escape_hstore(std::string& out, const char* str) {
-    out += "\"";
+static void add_escape_hstore(std::string* out, const char* str) {
+    *out += "\"";
 
     while (*str) {
         if (*str == '"') {
-            out += "\\\"";
+            *out += "\\\"";
         } else if (*str == '\\') {
-            out += "\\\\";
+            *out += "\\\\";
         } else {
-            out += *str;
+            *out += *str;
         }
         ++str;
     }
 
-    out += "\"";
+    *out += "\"";
 }
 
 bool ExportFormatPg::add_tags_hstore(const osmium::OSMObject& object) {
@@ -214,9 +215,9 @@ bool ExportFormatPg::add_tags_hstore(const osmium::OSMObject& object) {
     for (const auto& tag : object.tags()) {
         if (options().tags_filter(tag)) {
             has_tags = true;
-            add_escape_hstore(data, tag.key());
+            add_escape_hstore(&data, tag.key());
             data += "=>";
-            add_escape_hstore(data, tag.value());
+            add_escape_hstore(&data, tag.value());
             data += ',';
         }
     }
