@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,14 +20,15 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include "../util.hpp"
 #include "export_format_text.hpp"
 
+#include "../util.hpp"
+
 #include <osmium/io/detail/read_write.hpp>
 #include <osmium/io/detail/string_util.hpp>
 
-static constexpr const std::size_t initial_buffer_size = 1024 * 1024;
-static constexpr const std::size_t flush_buffer_size   =  800 * 1024;
+static constexpr const std::size_t initial_buffer_size = 1024UL * 1024UL;
+static constexpr const std::size_t flush_buffer_size   =  800UL * 1024UL;
 
 ExportFormatText::ExportFormatText(const std::string& /*output_format*/,
                                    const std::string& output_filename,
