@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "export_handler.hpp"
+
 #include "../exception.hpp"
 #include "../util.hpp"
 
@@ -109,14 +110,10 @@ void ExportHandler::way(const osmium::Way& way) {
         return;
     }
 
-    if (way.nodes().size() <= 1) {
-        if (m_show_errors) {
-            std::cerr << "Geometry error: Way with less than two nodes (id=" << way.id() << ")\n";
-        }
-        return;
-    }
-
     try {
+        if (way.nodes().size() <= 1) {
+            throw osmium::geometry_error{"Way with less than two nodes (id=" + std::to_string(way.id()) + ")"};
+        }
         if (!way.nodes().front().location() || !way.nodes().back().location()) {
             throw osmium::invalid_location{"invalid location"};
         }
@@ -137,14 +134,21 @@ void ExportHandler::area(const osmium::Area& area) {
         return;
     }
 
-    if (!area.from_way() || is_area(area.tags())) {
-        try {
-            m_handler->area(area);
-        } catch (const osmium::geometry_error& e) {
-            show_error(e);
-        } catch (const osmium::invalid_location& e) {
-            show_error(e);
+    if (area.from_way() && !is_area(area.tags())) {
+        return;
+    }
+
+    try {
+        const auto rings = area.num_rings();
+        if (rings.first == 0 && rings.second == 0) {
+            throw osmium::geometry_error{"Could not build area geometry"};
         }
+
+        m_handler->area(area);
+    } catch (const osmium::geometry_error& e) {
+        show_error(e);
+    } catch (const osmium::invalid_location& e) {
+        show_error(e);
     }
 }
 
