@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <string>
 
 std::string get_value_as_string(const rapidjson::Value& object, const char* key);
-std::size_t parse_polygon_array(const rapidjson::Value& value, osmium::memory::Buffer& buffer);
-std::size_t parse_multipolygon_array(const rapidjson::Value& value, osmium::memory::Buffer& buffer);
+std::size_t parse_polygon_array(const rapidjson::Value& value, osmium::memory::Buffer* buffer);
+std::size_t parse_multipolygon_array(const rapidjson::Value& value, osmium::memory::Buffer* buffer);
 
 /**
  * Gets areas from OSM files.
