@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "poly_file_parser.hpp"
+
 #include "../exception.hpp"
 #include "geometry_util.hpp"
-#include "poly_file_parser.hpp"
 
 #include <osmium/geom/coordinates.hpp>
 #include <osmium/memory/buffer.hpp>
@@ -30,6 +31,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include <algorithm>
 #include <fstream>
+#include <memory>
 #include <sstream>
 #include <string>
 #include <vector>
@@ -134,7 +136,7 @@ std::size_t PolyFileParser::operator()() {
         throw poly_error{std::string{"File '"} + m_file_name + "' is empty."};
     }
 
-    m_builder.reset(new osmium::builder::AreaBuilder{m_buffer});
+    m_builder = std::make_unique<osmium::builder::AreaBuilder>(m_buffer);
     while (m_line < m_data.size()) {
         parse_multipolygon();
     }
