@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "strategy_smart.hpp"
+
 #include "../util.hpp"
 
 #include <osmium/handler/check_order.hpp>
@@ -133,7 +134,7 @@ namespace strategy_smart {
 
     public:
 
-        explicit Pass1(Strategy& strategy) :
+        explicit Pass1(Strategy* strategy) :
             Pass(strategy) {
         }
 
@@ -141,9 +142,9 @@ namespace strategy_smart {
             m_check_order.node(node);
         }
 
-        void enode(extract_data& e, const osmium::Node& node) {
-            if (e.contains(node.location())) {
-                e.node_ids.set(node.positive_id());
+        void enode(extract_data* e, const osmium::Node& node) {
+            if (e->contains(node.location())) {
+                e->node_ids.set(node.positive_id());
             }
         }
 
@@ -151,10 +152,10 @@ namespace strategy_smart {
             m_check_order.way(way);
         }
 
-        void eway(extract_data& e, const osmium::Way& way) {
+        void eway(extract_data* e, const osmium::Way& way) {
             for (const auto& nr : way.nodes()) {
-                if (e.node_ids.get(nr.positive_ref())) {
-                    e.way_ids.set(way.positive_id());
+                if (e->node_ids.get(nr.positive_ref())) {
+                    e->way_ids.set(way.positive_id());
                     return;
                 }
             }
@@ -165,16 +166,16 @@ namespace strategy_smart {
             m_relations_map_stash.add_members(relation);
         }
 
-        void erelation(extract_data& e, const osmium::Relation& relation) {
+        void erelation(extract_data* e, const osmium::Relation& relation) {
             std::size_t wanted_members = 0;
             for (const auto& member : relation.members()) {
                 switch (member.type()) {
                     case osmium::item_type::node:
-                        if (e.node_ids.get(member.positive_ref())) {
+                        if (e->node_ids.get(member.positive_ref())) {
                             if (wanted_members == 0) {
-                                e.relation_ids.set(relation.positive_id());
+                                e->relation_ids.set(relation.positive_id());
                                 if (strategy().check_type(relation)) {
-                                    e.add_relation_members(relation);
+                                    e->add_relation_members(relation);
                                     return;
                                 }
                             }
@@ -182,11 +183,11 @@ namespace strategy_smart {
                         }
                         break;
                     case osmium::item_type::way:
-                        if (e.way_ids.get(member.positive_ref())) {
+                        if (e->way_ids.get(member.positive_ref())) {
                             if (wanted_members == 0) {
-                                e.relation_ids.set(relation.positive_id());
+                                e->relation_ids.set(relation.positive_id());
                                 if (strategy().check_type(relation)) {
-                                    e.add_relation_members(relation);
+                                    e->add_relation_members(relation);
                                     return;
                                 }
                             }
@@ -199,7 +200,7 @@ namespace strategy_smart {
             }
 
             if (strategy().check_members_count(relation.members().size(), wanted_members)) {
-                e.add_relation_members(relation);
+                e->add_relation_members(relation);
             }
         }
 
@@ -213,15 +214,15 @@ namespace strategy_smart {
 
     public:
 
-        explicit Pass2(Strategy& strategy) :
+        explicit Pass2(Strategy* strategy) :
             Pass(strategy) {
         }
 
-        void eway(extract_data& e, const osmium::Way& way) {
-            if (e.way_ids.get(way.positive_id()) ||
-                e.extra_way_ids.get(way.positive_id())) {
+        void eway(extract_data* e, const osmium::Way& way) {
+            if (e->way_ids.get(way.positive_id()) ||
+                e->extra_way_ids.get(way.positive_id())) {
                 for (const auto& nr : way.nodes()) {
-                    e.extra_node_ids.set(nr.ref());
+                    e->extra_node_ids.set(nr.ref());
                 }
             }
         }
@@ -232,28 +233,28 @@ namespace strategy_smart {
 
     public:
 
-        explicit Pass3(Strategy& strategy) :
+        explicit Pass3(Strategy* strategy) :
             Pass(strategy) {
         }
 
-        void enode(extract_data& e, const osmium::Node& node) {
-            if (e.node_ids.get(node.positive_id()) ||
-                e.extra_node_ids.get(node.positive_id())) {
-                e.write(node);
+        void enode(extract_data* e, const osmium::Node& node) {
+            if (e->node_ids.get(node.positive_id()) ||
+                e->extra_node_ids.get(node.positive_id())) {
+                e->write(node);
             }
         }
 
-        void eway(extract_data& e, const osmium::Way& way) {
-            if (e.way_ids.get(way.positive_id()) ||
-                e.extra_way_ids.get(way.positive_id())) {
-                e.write(way);
+        void eway(extract_data* e, const osmium::Way& way) {
+            if (e->way_ids.get(way.positive_id()) ||
+                e->extra_way_ids.get(way.positive_id())) {
+                e->write(way);
             }
         }
 
-        void erelation(extract_data& e, const osmium::Relation& relation) {
-            if (e.relation_ids.get(relation.positive_id()) ||
-                e.extra_relation_ids.get(relation.positive_id())) {
-                e.write(relation);
+        void erelation(extract_data* e, const osmium::Relation& relation) {
+            if (e->relation_ids.get(relation.positive_id()) ||
+                e->extra_relation_ids.get(relation.positive_id())) {
+                e->write(relation);
             }
         }
 
@@ -269,7 +270,7 @@ namespace strategy_smart {
         osmium::ProgressBar progress_bar{file_size * 3, display_progress};
 
         vout << "First pass (of three)...\n";
-        Pass1 pass1{*this};
+        Pass1 pass1{this};
         pass1.run(progress_bar, input_file, osmium::io::read_meta::no);
         progress_bar.file_done(file_size);
 
@@ -283,13 +284,13 @@ namespace strategy_smart {
 
         progress_bar.remove();
         vout << "Second pass (of three)...\n";
-        Pass2 pass2{*this};
+        Pass2 pass2{this};
         pass2.run(progress_bar, input_file, osmium::osm_entity_bits::way, osmium::io::read_meta::no);
         progress_bar.file_done(file_size);
 
         progress_bar.remove();
         vout << "Third pass (of three)...\n";
-        Pass3 pass3{*this};
+        Pass3 pass3{this};
         pass3.run(progress_bar, input_file);
 
         progress_bar.done();
