@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -48,7 +48,8 @@ namespace strategy_smart {
 
     class Strategy : public ExtractStrategy {
 
-        template<typename S, typename T> friend class ::Pass;
+        template <typename S, typename T>
+        friend class ::Pass;
         friend class Pass1;
 
         using extract_data = ExtractData<Data>;
