@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include "exception.hpp"
 #include "util.hpp"
 
+#include "exception.hpp"
+
 #include <osmium/io/file.hpp>
 #include <osmium/osm/area.hpp>
 #include <osmium/tags/tags_filter.hpp>
@@ -172,7 +173,7 @@ osmium::StringMatcher get_string_matcher(std::string string) {
     return osmium::StringMatcher::substring{s};
 }
 
-osmium::TagMatcher get_tag_matcher(const std::string& expression, bool *has_value_matcher) {
+osmium::TagMatcher get_tag_matcher(const std::string& expression, bool* has_value_matcher) {
     const auto op_pos = expression.find('=');
     if (op_pos == std::string::npos) {
         if (has_value_matcher) {
@@ -265,3 +266,11 @@ bool ends_with(const std::string& str, const std::string& suffix) {
                        suffix.c_str()) == 0;
 }
 
+std::size_t show_mbytes(std::size_t value) noexcept {
+    return value / (1024UL * 1024UL);
+}
+
+double show_gbytes(std::size_t value) noexcept {
+    return static_cast<double>(show_mbytes(value)) / 1000; // NOLINT(bugprone-integer-division)
+}
+
