@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2021  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2022  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -45,11 +45,13 @@ osmium::osm_entity_bits::type get_types(const std::string& str);
 std::pair<osmium::osm_entity_bits::type, std::string> get_filter_expression(const std::string& str);
 void strip_whitespace(std::string& string);
 osmium::StringMatcher get_string_matcher(std::string string);
-osmium::TagMatcher get_tag_matcher(const std::string& expression, bool *has_value_matcher = nullptr);
+osmium::TagMatcher get_tag_matcher(const std::string& expression, bool* has_value_matcher = nullptr);
 void initialize_tags_filter(osmium::TagsFilter& tags_filter, bool default_result, const std::vector<std::string>& strings);
 osmium::Box parse_bbox(const std::string& str, const std::string& option_name);
 osmium::item_type parse_item_type(const std::string& t);
 const char* object_type_as_string(const osmium::OSMObject& object) noexcept;
 bool ends_with(const std::string& str, const std::string& suffix);
+std::size_t show_mbytes(std::size_t value) noexcept;
+double show_gbytes(std::size_t value) noexcept;
 
 #endif // UTIL_HPP
