@@ -8,31 +8,31 @@ TEST_CASE("diff") {
     CommandDiff cmd{factory};
 
     SECTION("no arguments - need exactly two arguments") {
-        REQUIRE_THROWS_AS(cmd.setup({}), const argument_error&);
+        REQUIRE_THROWS_AS(cmd.setup({}), argument_error);
     }
 
     SECTION("one argument - need exactly two arguments") {
-        REQUIRE_THROWS_AS(cmd.setup({"x"}), const argument_error&);
+        REQUIRE_THROWS_AS(cmd.setup({"x"}), argument_error);
     }
 
     SECTION("three arguments - need exactly two arguments") {
-        REQUIRE_THROWS_AS(cmd.setup({"x", "y", "z"}), const argument_error&);
+        REQUIRE_THROWS_AS(cmd.setup({"x", "y", "z"}), argument_error);
     }
 
     SECTION("quiet with output parameter -o") {
-        REQUIRE_THROWS_AS(cmd.setup({"-q", "-o", "file"}), const argument_error&);
+        REQUIRE_THROWS_AS(cmd.setup({"-q", "-o", "file"}), argument_error);
     }
 
     SECTION("quiet with output parameter -f") {
-        REQUIRE_THROWS_AS(cmd.setup({"-q", "-f", "opl"}), const argument_error&);
+        REQUIRE_THROWS_AS(cmd.setup({"-q", "-f", "opl"}), argument_error);
     }
 
     SECTION("parameter --fsync") {
-        REQUIRE_THROWS_AS(cmd.setup({"--fsync"}), const boost::program_options::unknown_option&);
+        REQUIRE_THROWS_AS(cmd.setup({"--fsync"}), boost::program_options::unknown_option);
     }
 
     SECTION("quiet with output parameter -O") {
-        REQUIRE_THROWS_AS(cmd.setup({"-q", "-O"}), const argument_error&);
+        REQUIRE_THROWS_AS(cmd.setup({"-q", "-O"}), argument_error);
     }
 
 }
